@@ -28,6 +28,7 @@ from dstk.utils.errors import TrainError
 # BaseEnvironnement
 PyTorchEstimators = NewType('PyTorchEstimators', Any)
 
+
 @dataclass
 class FitState:
     r"""
@@ -35,19 +36,19 @@ class FitState:
 
     Parameters
     ----------
-    id_process : str
+    id_process: str
         Identifiant du processus.
-    nb_epoch : int
+    nb_epoch: int
         Nombre d'époques du processus d'apprentissage.
-    epoch_max_steps : int
+    epoch_max_steps: int
         Nombre d'itérations à l'intérieur d'une époque.
-    eval_max_steps : int
+    eval_max_steps: int
         Nombre d'itérations à l'intérieur de la phase d'évaluation.
-    loss_name : str
+    loss_name: str
         Nom de la fonction coût.
-    n_iter : int (=0)
+    n_iter: int (=0)
         Numéro de l'itération en cours.
-    n_epoch : int (=0)
+    n_epoch: int (=0)
         Numéro de l'époque en cours.
     """
     id_process: str
@@ -58,6 +59,7 @@ class FitState:
     n_iter: int = 0
     n_epoch: int = 0
 
+
 @dataclass
 class FitControl:
     r"""
@@ -65,27 +67,27 @@ class FitControl:
 
     Parameters
     ----------
-    training_stop : bool (=False)
+    training_stop: bool (=False)
         Booléen indiquant si le processus d'apprentissage doit stopper ou non.
-    diverge : bool (=False)
+    diverge: bool (=False)
         Booléen indiquant si le processus d'apprentissage a divergé ou non.
     target_field: str (='target')
         Nom du champ de la target dans le dictionnaire d'échange.
     target_checktensor: bool (=True)
             Permet de checker si le champs target est un tensor, sinon le
             converti. Si False ne checke pas si le champs est un tensor.
-    device : device (=device('cpu:0'))
+    device: device (=device('cpu:0'))
         Device sur le quel le modèle exécute son entrainement.
-    mixed_type_gpu : bool (=False)
+    mixed_type_gpu: bool (=False)
         Indicateur pour que l'apprentissage utiliser les types mixé fp16 et 
         fp32 via l'API AMP (Automatic Mixed Precision).
-    grad_scaler : GradScaler (=GradScaler(enabled=False))
-    gradient_accumulation : Optional[int]
+    grad_scaler: GradScaler (=GradScaler(enabled=False))
+    gradient_accumulation: Optional[int]
         Nombre d'itération pour l'accumulation du gradient. Si None ou < 1 
         mise du nombre d'itération à 1.
-    max_gradient_norm : Optional[float]
+    max_gradient_norm: Optional[float]
         Valeur max du gradient pour son clipping.
-    expected_key_to_device : List[str] (=list())
+    expected_key_to_device: List[str] (=list())
         Liste des variables à exclure vers la charge sur GPU.
     """
     training_stop: bool = False
@@ -106,6 +108,7 @@ class FitControl:
         ):
             self.gradient_accumulation = 1
 
+
 class CallbackInterface:
     r"""
     Classe interface permettant de définir des modules utilisés durant 
@@ -162,21 +165,22 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
 
+
     @abstractmethod
     def end_train(
         self,
@@ -193,17 +197,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -223,17 +227,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -253,17 +257,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -283,17 +287,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -313,17 +317,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -343,17 +347,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -373,17 +377,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -403,17 +407,17 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         """
         pass
@@ -434,23 +438,24 @@ class CallbackInterface:
 
         Parameters
         ----------
-        model : PyTorchEstimators
+        model: PyTorchEstimators
             Référence au modèle sur lequel l'apprentissage se déroule.
-        data : Iterable
+        data: Iterable
             Data sous forme de liste ou d'un DataLoader.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        res_loss : float
+        res_loss: float
             Valeur de la fonction coût.
         exeption: Exception
             Type de l'exeption renvoyé.
         """
         pass
 
+
 class CallbackHandler(CallbackInterface):
     r"""
     Classe handler permettant de gérer simplement une liste de callbacks.
@@ -656,6 +661,7 @@ class CallbackHandler(CallbackInterface):
         for cb in self.callbacks:
             getattr(cb, step)(model, data, state, control, res_loss, **kwargs);
 
+
 @dataclass
 class PrintCallback(CallbackInterface):
     r"""
@@ -692,6 +698,7 @@ class PrintCallback(CallbackInterface):
             f"Apprentissage arrêté du à l'error: {exception}"
         )
 
+
 @dataclass
 class Backup(CallbackInterface, metaclass=SuperclassDocstring):
     r"""
@@ -704,19 +711,23 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
 
     Parameters
     ----------
-    path_backup : Optional[str]
-        Chemin de backup. Si None utilise le répertoire dans le quel est 
-        lancé l'instance Python. Le nom du backup est l'id_process.
-    n_iter : int (=1_000)
+    path_backup: str
+        Chemin de backup. Le nom du backup est : {id_process}_{n_iter}.bck
+    n_iter: int (=1_000)
         Nombre d'itération entre deux backup.
+    erase_folder: bool (=True)
+        Effase le répertoire une fois l'entrainement fini ou arrêté.
+    nb_backup: Optinal[int]
+        Nombre de backup préservé. Si None préserve tous les backups.
     """
-    path_backup: Optional[str] = None
+    path_backup: str = './backup'
     n_iter: int = 1_000
+    erase_folder: bool = True
+    nb_backup: Optional[int] = None
 
     def __post_init__(self):
-        self.path_backup = (
-            self.path_backup if self.path_backup else os.getcwd()
-        )
+        if not os.path.exists(self.path_backup):
+            os.makedirs(self.path_backup)
 
     def end_step(
         self,
@@ -735,8 +746,23 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
                     state,
                     control
                 ],
-                open(f"{self.path_backup}/{state.id_process}.bck", "wb")
+                open(
+                    f"{self.path_backup}/{state.id_process}_{state.n_iter}.bck",
+                    "wb"
+                )
             )
+        if self.nb_backup:
+            files = sorted(
+                list(
+                    filter(
+                        lambda x: x.startswith(state.id_process),
+                        os.listdir(f"{self.path_backup}/")
+                    )
+                ),
+                reverse=True
+            )[self.nb_backup:]
+            for file in files:
+                os.remove(f"{self.path_backup}/{file}")
 
     def end_train(
         self,
@@ -747,7 +773,15 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        os.remove(f"{self.path_backup}/{state.id_process}.bck")
+        if self.erase_folder:
+            files = list(
+                filter(
+                    lambda x: x.startswith(state.id_process),
+                    os.listdir(f"{self.path_backup}/")
+                )
+            )
+            for file in files:
+                os.remove(f"{self.path_backup}/{file}")
 
     def error_interrupt(
         self,
@@ -759,8 +793,16 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
         exception: Exception,
         **kwargs
     ) -> None:
-        if isinstance(exception, KeyboardInterrupt):
-            os.remove(f"{self.path_backup}/{state.id_process}.bck")
+        if isinstance(exception, KeyboardInterrupt) and self.erase_folder:
+            files = list(
+                filter(
+                    lambda x: x.startswith(state.id_process),
+                    os.listdir(f"{self.path_backup}/")
+                )
+            )
+            for file in files:
+                os.remove(f"{self.path_backup}/{file}")
+
 
 @dataclass
 class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
@@ -772,21 +814,21 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
 
     Parameters
     ----------
-    path_early_stopping : Optional[str] (=None)
+    path_early_stopping: Optional[str] (=None)
         Chemin (absolu ou relatif) où le modèle par early stopping sera 
         préservé. Si None, les paramètres seront sauvegardés en RAM. 
         Cependant il est déconseillé de le faire car les réseaux profonds 
         peuvent être très gourmands en espace RAM. De même, en cas de 
         plantage, un état du réseau pouvant être satisfaisant est préservé 
         sur le disque dur.
-    early_stopping_rounds : Optional[int] (=None)
+    early_stopping_rounds: Optional[int] (=None)
         Nombre d'itérations (epoch) sans amélioration. Si None pas de 
         early stopping, le modèle sauvegardé sera donc le meilleur modèle 
         durant l'ensemble des époques.
-    minimize : bool (=True)
+    minimize: bool (=True)
         Indique si l'objectif est de minimiser la fonciton coût (True) ou 
         de la maximiser (False).
-    verbose : bool (=False)
+    verbose: bool (=False)
         Indique les informations de gain de performance.
     """
     path_early_stopping: Optional[str] = None
@@ -799,6 +841,9 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
         self.iter_stop: int = 0
         self.best_epoch: int = 0
         self.best_iter: int = 0
+        if self.path_early_stopping:
+            if os.path.exists(self.path_early_stopping):
+                os.makedirs(self.path_early_stopping)
 
     def end_eval(
         self,
@@ -893,28 +938,30 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
                 **kwargs
             )
 
+
 @dataclass
 class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
     r"""
-    Classe de gestion du learning rate. Ce callback ne met à jour le 
-    learning rate que durant les périodes d'évaluation.
+    Classe de gestion du learning rate. Possibilité de réaliser la mise à jour
+    le learning rate soit durant la phase d'évaluation (by_step=False) ou
+    a chaque étape (by_step=True).
 
     Parameters
     ----------
-    lr_scheduler : Callable
+    lr_scheduler: Callable
         Classe non instanciée d'un planificateur de learning rate. Les 
         planificateurs de learning rate se situent dans 
         torch.optim.lr_scheduler et permettent de contrôler le lerning rate au 
         travers des époques.
-    scheduler_kwargs : Dict[str, Any] (=dict())
+    scheduler_kwargs: Dict[str, Any] (=dict())
         Argument d'entrée de scheduler (en dehors de l'optimizer qui doit 
         être inclu à l'intérieur de la classe du réseau). Explicitement 
         l'instanciation sera faite dans la méthode fit sera du type :
             lr_scheduler(self.optimizer, **scheduler_kwargs)
-    by_step : bool (=False)
+    by_step: bool (=False)
         Détermine si le LRScheduler est appelé à la fin de la phase 
         d'évaluation (=False) ou à la fin de chaque step de train (=True).
-    verbose : bool (=True)
+    verbose: bool (=False)
         Permet d'affichier le lerning rate initial et les évolutions du 
         learning rate quand ils ont lieu. Par défaut cette argument est 
         à True car la sortie n'est pas très verbeuse.
@@ -931,7 +978,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
     lr_scheduler: Callable
     scheduler_kwargs: Dict[str, Any] = field(default_factory=dict)
     by_step: bool = False
-    verbose: bool = True
+    verbose: bool = False
 
     def __post_init__(self):
         if self.lr_scheduler.__name__ == "ReduceLROnPlateau" and self.by_step:
@@ -1011,20 +1058,21 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
                     )
                     self.old_lr = lr
 
+
 @dataclass
 class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
     r"""
     Barre de progression TQDM sur les époques et les évaluations.
 
-    mininterval : float
+    mininterval: float (=0.5)
         Temps en seconde entre les intervalles de raffraichissement de la 
-        barre de progression. Par défaut 0.5.
-    bar_epoch : bool
+        barre de progression.
+    bar_epoch: bool (=True)
         Indication si la barre de progression sera exécutée sur une époque 
-        d'apprentissage durant l'entrainement. Par défaut True.
-    bar_eval : bool
+        d'apprentissage durant l'entrainement.
+    bar_eval: bool (=False)
         Indication si la barre de progression sera exécutée sur une époque 
-        d'évaluation durant l'entrainement. Par défaut False.
+        d'évaluation durant l'entrainement.
     """
     mininterval: float = 0.5
     bar_epoch: bool = True
@@ -1119,6 +1167,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         if self.bar_eval:
             self.eval_bar.update(1)
 
+
 class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
     r"""
     Callback permettant de faire le suivit de l'apprentissage via Tensorboard.
@@ -1127,10 +1176,10 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
 
     Parameters
     ----------
-    log_dir : Optional[str]
+    log_dir: Optional[str]
         Chemin de sauvegarde des données Tensorboard, par défaut 
         runs/CURRENT_DATETIME_HOSTNAME.
-    comment : str (='')
+    comment: str (='')
         Concatène un commentaire au log_dir. Si log_dir est renseigné cette 
         argument n'est pas pris en compte.
     purge_step: Optional[int]
@@ -1138,11 +1187,11 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
         events whose global_step larger or equal to T will be purged and 
         hidden from TensorBoard. Note that crashed and resumed experiments 
         should have the same log_dir.
-    max_queue : int (=10)
+    max_queue: int (=10)
         Taille de la queue en attente avec de mettre sur le disque.
-    flush_secs : int (=120)
+    flush_secs: int (=120)
         Temps entre deux mise sur le disque.
-    filename_suffix : str (='')
+    filename_suffix: str (='')
         Ajout d'un suffix au log_dir.
     """
     def __init__(
@@ -1207,6 +1256,7 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
     ) -> None:
         self.writer.add_scalar('Loss/eval', res_loss, state.n_iter)
 
+
 DEFAULT_CALLBACKS = [
     ProgressBarCallback(),
     PrintCallback(),
