@@ -10,6 +10,7 @@ from ._statistique import weighted_avg_and_std
 from ._transpose import (list_of_dict_2_dict_of_list,
                          dict_of_list_2_list_of_dict)
 from ._rolling import RollingWindow
+from ._trie import Trie
 
 __all__ = (
     'Chunker',
@@ -24,5 +25,6 @@ __all__ = (
     'weighted_avg_and_std',
     'list_of_dict_2_dict_of_list',
     'dict_of_list_2_list_of_dict',
-    'RollingWindow'
+    'RollingWindow',
+    'Trie'
 )
